
# coding: utf-8

# In[1]:

from datetime import date, datetime, timedelta
import sys, time
import visa
import math
# if not using IPython leave out the next line.
get_ipython().magic('matplotlib inline')
import matplotlib.pyplot as plt
import numpy as np


# In[2]:

def init_instr(dmm):
    dmm.write('*CLS')
    dmm.write('*RST')
    dmm.write('INP:IMP:AUTO ON')
    dmm.write('DISP:STAT ON')
    temp = round(float(dmm.query('SYST:TEMP?')), 2)
    print('\nStart temperature: {0:.2f} deg C.\n'.format(temp))
    
def meas(dmm, ran, nplc, samples): 
    dmm.write('CONF:VOLT:DC {0}'.format(ran))
    dmm.write('VOLT:DC:NPLC {0}'.format(nplc))    
    dmm.write('TRIG:SOUR BUS')
    dmm.write('SAMP:COUN {0}'.format(str(samples)))
    dmm.write(':INIT')
    dmm.write('*TRG')
    dmm.write('*WAI')
    values = dmm.query('FETC?').split(',') 
    list_of_floats = list(map(float, values))
    return np.array(list_of_floats, dtype = 'float')
 
def reset_instr(dmm):
    temp = round(float(dmm.query('SYST:TEMP?')), 2)
    print('\nEnd temperature: {0:.2f} deg C.\n'.format(temp))
    dmm.write('*CLS')
    dmm.write('*RST')
    dmm.write('SYST:LOC')
    
def meas_loop(dmm, count_in_plc):  
    print(75*'=')
    print(' Range    NPLC   Mean [uV]  Std. dev. [uV]  Samples   PPM of R')     
    ppm_of_range = [] 
    
    for r in (0.1, 1, 10, 100, 1000):    
        por = []
        ppm_of_range.append(por)
        print(75*'-')
        
        for nplc in (0.02, 0.06, 0.2, 1, 10, 100):  
            samples=int(count_in_plc/nplc)
            data = meas(dmm, ran=str(r), nplc=str(nplc), samples=samples)
                                    
            mean = np.mean(data)*1e6
            std = np.std(data)*1e6
        
            ppm_range = std/r
            por.append(ppm_range)
            print('{0:6.1f} {1:7.2f} {2:11.2f} {3:15.2f} {4:8} {5:10.3f}'.format(r, nplc, mean, std, samples, ppm_range))
   
    print(75*'=')  
    return ppm_of_range     


# In[3]:

def plot(data, count_in_plc):
    n = (0.02, 0.06, 0.2, 1, 10, 100)
    fig, ax = plt.subplots(1, 1, figsize=(14, 7))  
    ax.grid(True)
    for i in range(5):
        ax.loglog(n, data[i], linewidth=0.5)  
    ax.grid(True, which="both")   # ls="-"
    ax.set_title('Keysight 34470A - Noise floor, zero volts input, auto zero on, num_of_plc={0}'.format(count_in_plc))
    ax.set_xlabel('NPLC')
    ax.set_ylabel('Noise Std. dev. [ppm of range]')
    
def main(count_in_plc):        
    try:
        rm = visa.ResourceManager()     
        resources = rm.list_resources()  
        print(resources)   
        dmm = rm.open_resource('TCPIP0::K-000000-00000::inst0::INSTR')
        dmm.timeout = 10000000       
        
        try: 
            with dmm.ignore_warning(visa.constants.VI_SUCCESS_MAX_CNT):
                init_instr(dmm)  
                data = meas_loop(dmm, count_in_plc)   
                reset_instr(dmm)
                plot(data, count_in_plc)
        except Exception as err:
            print(err)
        finally:
            dmm.close()
         
    except Exception as err:
        print('Visa error: '+str(err))


# In[4]:

main(count_in_plc=500)


# In[ ]:

# this loop takes a lot of time to complete ;-)
for plc in (100, 200, 500, 1000, 2000, 5000, 10000, 50000, 100000):
    main(plc)
    

